@@ -30,6 +30,7 @@ use vars qw( @ISA @EXPORT );
     apache_request
     base_server
     cast_custom_error
+    consume_post_body
     declined_response
     dispatch_location
     engine
@@ -48,8 +49,10 @@ use vars qw( @ISA @EXPORT );
     get_cached_config
     get_config
     get_dbh
+    get_post_body
     header_in
     header_out
+    hostname
     is_status_declined
     port
     print_output
@@ -88,7 +91,10 @@ sub file_upload {
     my $filename = $upload->filename;
     $filename =~ s/\\/\//g;
     
-    my( $name, $path, $suffix ) = fileparse( $filename, qr/\.[^.]*/ ); 
+    my( $name, $path, $suffix ) = fileparse( 
+		$filename, 
+		qr/\.(tar\.gz$|[^.]*)/ 
+	);  
     
     return( {
         unique_key => time . rand( 6 ),
@@ -133,14 +139,19 @@ sub cast_custom_error {
 #-------------------------------------------------
 sub apache_param_hash {
     my( $self, $req ) = @_;
+
+    my $hash = {};
     
-    my $params = {};
-    foreach my $p ( $req->param ) {
-        $params->{$p} = $req->param( $p );
-    }
-
-    return( $params );
+    my @param_names = $req->param;
+    foreach my $p ( @param_names ) {
+        my @values = $req->param( $p );
+        
+        $hash->{$p} = join( "\0", @values );
 
+    }   
+     
+    return( $hash );
+    
 } # end: apache_param_hash
 
 #-------------------------------------------------
@@ -148,7 +159,7 @@ sub apache_param_hash {
 #-------------------------------------------------
 sub apache_request {
     my( $self, $r ) = @_;
-    
+
     return( 
         $r ? Apache2::Request->new( $r, POST_MAX => $self->post_max )
         : Apache2::Request->new( $self->r, POST_MAX => $self->post_max ) );
@@ -167,6 +178,40 @@ sub base_server {
 
 } # end base_server
 
+#-------------------------------------------------
+# $self->consume_post_body( $r )
+#-------------------------------------------------
+sub consume_post_body {
+    my $self = shift;
+    my $r    = shift;
+
+    my $content_length = $r->headers_in->{'Content-length'};
+
+    return unless $content_length;
+
+    $content_length    = 1e6 if $content_length > 1e6; # limit to ~ 1Meg
+
+    my ( $content, $buffer );
+
+    while ( $r->read( $buffer, $content_length ) ) {
+        $content .= $buffer;
+    }
+
+    $self->{__POST_BODY__} = $content;
+}
+
+#-------------------------------------------------
+# $self->hostname( $r )
+#-------------------------------------------------
+sub hostname {
+    my( $self, $r ) = ( shift, shift );
+
+    return( 
+        $r ? $r->hostname
+        : $self->r->hostname );
+    
+} # end hostname
+
 #-------------------------------------------------
 # $self->declined_response( )
 #-------------------------------------------------
@@ -249,7 +294,7 @@ sub fish_method {
     my $self = shift;
 
     return $self->r()->method;
-} # END fish_uri
+} # END fish_method
 
 #-------------------------------------------------
 # $self->fish_path_info( )
@@ -275,7 +320,7 @@ sub fish_uri {
 sub fish_user {
     my $self = shift;
 
-    return $self->r()->user;
+    return $self->user() || $self->r()->user;
 } # END fish_uri
 
 #-------------------------------------------------
@@ -317,21 +362,67 @@ sub get_config {
 
     my $conf;
     my $cached    = 0;
-    my $location  = $self->location;
-    $conf         = $self->get_cached_config( $instance, $location );
+    my $location  = '';
+    
+    eval {
+        $location = $self->location;
+    };
+ 
+    $conf = $self->get_cached_config( $instance, $location );
 
-    $cached++ if ( defined $conf );
+    if ( defined $conf ) {
+        return $conf;
+    }
+ 
+    my $gantry_cache     = 0;
+    my $gantry_cache_key = '';
+    my $gantry_cache_hit = 0;
+    eval { 
+        ++$gantry_cache if $self->cache_inited();
+    };
+    
+    # are we using gantry cache ?
+    if ( $gantry_cache ) {
 
+        # blow the gantry conf cache when server starts
+        if ( $self->engine_cycle() == 1 ) {
+            
+            foreach my $key ( @{ $self->cache_keys() } ) {
+                my @a = split( ':', $key );                
+                if ( $a[0] eq 'gantryconf' ) {
+                    $self->cache_del( $key );
+                }
+            }
+        }
+                
+        # build cache key
+        $gantry_cache_key = join( ':',
+            "gantryconf",
+            ( $self->namespace() || '' ),
+            $instance,
+            $location
+        );
+    
+        $conf = $self->cache_get( $gantry_cache_key );
+        
+        ++$gantry_cache_hit if defined $conf;
+    }   
+     
     $conf ||= Gantry::Conf->retrieve(
         {
             instance    => $instance, 
             config_file => $file,
-            location    => $self->location()
+            location    => $location
         }
     );
 
-    $self->set_cached_config( $instance, $location, $conf )
-            if ( not $cached and defined $conf );
+    if ( defined $conf ) {        
+        $self->set_cached_config( $instance, $location, $conf );
+    
+        if ( $gantry_cache && ! $gantry_cache_hit ) {
+            $self->cache_set( $gantry_cache_key, $conf );
+        }
+    } 
 
     return $conf;
 
@@ -343,8 +434,9 @@ sub get_config {
 sub get_cached_config {
     my $self     = shift;
     my $instance = shift;
+    my $location = shift;
 
-    return $self->r()->pnotes( "conf_$instance" );
+    return $self->r()->pnotes( "conf_${instance}_${location}" );
 }
 
 #-------------------------------------------------
@@ -353,10 +445,10 @@ sub get_cached_config {
 sub set_cached_config {
     my $self     = shift;
     my $instance = shift;
-    shift;  # ignore location, cache for one request only
+    my $location = shift;  
     my $conf     = shift;
 
-    $self->r()->pnotes( "conf_$instance", $conf );
+    $self->r()->pnotes( "conf_${instance}_${location}", $conf );
 }
 
 #-------------------------------------------------
@@ -366,6 +458,15 @@ sub get_dbh {
     return Gantry::Utils::DBConnHelper::MP20->get_dbh;
 }
 
+#-------------------------------------------------
+# $self->get_post_body( )
+#-------------------------------------------------
+sub get_post_body {
+    my $self = shift;
+
+    return $self->{__POST_BODY__};
+}
+
 #-------------------------------------------------
 # $self->header_in( )
 #-------------------------------------------------
@@ -574,6 +675,16 @@ libapreq(3) manpage for more details.
 Returns the physical server this connection came in 
 on (main server or vhost).
 
+=item $self->consume_post_body
+
+This must be used by a plugin at the pre_init phase.  It takes all of the
+data from the body of the HTTP POST request, storing it for retrieval
+via C<get_post_body>.  You cannot mix this with regular form handling.
+
+=item $self->hostname
+
+Returns the virtual server name 
+
 =item $self->dispatch_location
 
 Returns the tail of the uri specific to the current location, i.e.:
@@ -649,6 +760,11 @@ set receives those plus the conf hash it should cache.
 Returns the current regular database connection if one is available
 or undef otherwise.
 
+=item $self->get_post_body
+
+If C<consume_post_body> was used by a plugin during the pre_init phase,
+this method returns the consumed body of the HTTP POST request.
+
 =item $self->header_in
 
 The $r->headers_in method will return a %hash of client request headers. 
