@@ -15,6 +15,7 @@ our @EXPORT = qw(
     cache_set
     cache_keys
     cache_init
+    cache_purge
     cache_handle
     cache_inited
     cache_expires
@@ -137,6 +138,14 @@ sub cache_del {
     
 }
 
+sub cache_purge {
+    my ($gobj) = @_;
+    
+    my $handle = $gobj->cache_handle();
+    $handle->purge();
+
+}
+
 1;
 
 __END__
